@@ -36,18 +36,36 @@
 #include "../cliprdr_common.h"
 
 #ifdef WITH_DEBUG_CLIPRDR
-static const char* const CB_MSG_TYPE_STRINGS[] = { "",
-	                                               "CB_MONITOR_READY",
-	                                               "CB_FORMAT_LIST",
-	                                               "CB_FORMAT_LIST_RESPONSE",
-	                                               "CB_FORMAT_DATA_REQUEST",
-	                                               "CB_FORMAT_DATA_RESPONSE",
-	                                               "CB_TEMP_DIRECTORY",
-	                                               "CB_CLIP_CAPS",
-	                                               "CB_FILECONTENTS_REQUEST",
-	                                               "CB_FILECONTENTS_RESPONSE",
-	                                               "CB_LOCK_CLIPDATA",
-	                                               "CB_UNLOCK_CLIPDATA" };
+static const char* CB_MSG_TYPE_STRINGS(UINT32 type)
+{
+	switch (type)
+	{
+		case CB_MONITOR_READY:
+			return "CB_MONITOR_READY";
+		case CB_FORMAT_LIST:
+			return "CB_FORMAT_LIST";
+		case CB_FORMAT_LIST_RESPONSE:
+			return "CB_FORMAT_LIST_RESPONSE";
+		case CB_FORMAT_DATA_REQUEST:
+			return "CB_FORMAT_DATA_REQUEST";
+		case CB_FORMAT_DATA_RESPONSE:
+			return "CB_FORMAT_DATA_RESPONSE";
+		case CB_TEMP_DIRECTORY:
+			return "CB_TEMP_DIRECTORY";
+		case CB_CLIP_CAPS:
+			return "CB_CLIP_CAPS";
+		case CB_FILECONTENTS_REQUEST:
+			return "CB_FILECONTENTS_REQUEST";
+		case CB_FILECONTENTS_RESPONSE:
+			return "CB_FILECONTENTS_RESPONSE";
+		case CB_LOCK_CLIPDATA:
+			return "CB_LOCK_CLIPDATA";
+		case CB_UNLOCK_CLIPDATA:
+			return "CB_UNLOCK_CLIPDATA";
+		default:
+			return "UNKNOWN";
+	}
+}
 #endif
 
 CliprdrClientContext* cliprdr_get_client_interface(cliprdrPlugin* cliprdr)
@@ -119,6 +137,9 @@ static void cliprdr_print_general_capability_flags(UINT32 flags)
 	if (flags & CB_CAN_LOCK_CLIPDATA)
 		WLog_INFO(TAG, "\tCB_CAN_LOCK_CLIPDATA");
 
+	if (flags & CB_HUGE_FILE_SUPPORT_ENABLED)
+		WLog_INFO(TAG, "\tCB_HUGE_FILE_SUPPORT_ENABLED");
+
 	WLog_INFO(TAG, "}");
 }
 #endif
@@ -157,6 +178,7 @@ static UINT cliprdr_process_general_capability(cliprdrPlugin* cliprdr, wStream*
 	cliprdr->streamFileClipEnabled = (generalFlags & CB_STREAM_FILECLIP_ENABLED);
 	cliprdr->fileClipNoFilePaths = (generalFlags & CB_FILECLIP_NO_FILE_PATHS);
 	cliprdr->canLockClipData = (generalFlags & CB_CAN_LOCK_CLIPDATA);
+	cliprdr->hasHugeFileSupport = (generalFlags & CB_HUGE_FILE_SUPPORT_ENABLED);
 	cliprdr->capabilitiesReceived = TRUE;
 
 	if (!context->custom)
@@ -441,7 +463,7 @@ static UINT cliprdr_order_recv(cliprdrPlugin* cliprdr, wStream* s)
 
 #ifdef WITH_DEBUG_CLIPRDR
 	WLog_DBG(TAG, "msgType: %s (%" PRIu16 "), msgFlags: %" PRIu16 " dataLen: %" PRIu32 "",
-	         CB_MSG_TYPE_STRINGS[msgType], msgType, msgFlags, dataLen);
+	         CB_MSG_TYPE_STRINGS(msgType), msgType, msgFlags, dataLen);
 	winpr_HexDump(TAG, WLOG_DEBUG, Stream_Buffer(s), dataLen + 8);
 #endif
 
@@ -570,12 +592,15 @@ static UINT cliprdr_client_capabilities(CliprdrClientContext* context,
 	if (!cliprdr->fileClipNoFilePaths)
 		flags &= ~CB_FILECLIP_NO_FILE_PATHS;
 	if (!cliprdr->canLockClipData)
-		flags &= CB_CAN_LOCK_CLIPDATA;
+		flags &= ~CB_CAN_LOCK_CLIPDATA;
+	if (!cliprdr->hasHugeFileSupport)
+		flags &= ~CB_HUGE_FILE_SUPPORT_ENABLED;
 
 	cliprdr->useLongFormatNames = flags & CB_USE_LONG_FORMAT_NAMES;
 	cliprdr->streamFileClipEnabled = flags & CB_STREAM_FILECLIP_ENABLED;
 	cliprdr->fileClipNoFilePaths = flags & CB_FILECLIP_NO_FILE_PATHS;
 	cliprdr->canLockClipData = flags & CB_CAN_LOCK_CLIPDATA;
+	cliprdr->hasHugeFileSupport = flags & CB_HUGE_FILE_SUPPORT_ENABLED;
 
 	Stream_Write_UINT32(s, flags); /* generalFlags */
 	WLog_Print(cliprdr->log, WLOG_DEBUG, "ClientCapabilities");
@@ -774,6 +799,18 @@ cliprdr_client_file_contents_request(CliprdrClientContext* context,
 	wStream* s;
 	cliprdrPlugin* cliprdr = (cliprdrPlugin*)context->handle;
 
+	if (!cliprdr)
+		return ERROR_INTERNAL_ERROR;
+
+	if (!cliprdr->hasHugeFileSupport)
+	{
+		if (((UINT64)fileContentsRequest->cbRequested + fileContentsRequest->nPositionLow) >
+		    UINT32_MAX)
+			return ERROR_INVALID_PARAMETER;
+		if (fileContentsRequest->nPositionHigh != 0)
+			return ERROR_INVALID_PARAMETER;
+	}
+
 	s = cliprdr_packet_file_contents_request_new(fileContentsRequest);
 
 	if (!s)
