@@ -1194,6 +1194,14 @@ int rdp_client_transition_to_state(rdpRdp* rdp, int state)
 
 		case CONNECTION_STATE_ACTIVE:
 			rdp->state = CONNECTION_STATE_ACTIVE;
+			{
+				ActivatedEventArgs activatedEvent;
+				rdpContext* context = rdp->context;
+				EventArgsInit(&activatedEvent, "libfreerdp");
+				activatedEvent.firstActivation = !rdp->deactivation_reactivation;
+				PubSub_OnActivated(context->pubSub, context, &activatedEvent);
+			}
+
 			break;
 
 		default:
@@ -1201,6 +1209,15 @@ int rdp_client_transition_to_state(rdpRdp* rdp, int state)
 			break;
 	}
 
+	{
+		ConnectionStateChangeEventArgs stateEvent;
+		rdpContext* context = rdp->context;
+		EventArgsInit(&stateEvent, "libfreerdp");
+		stateEvent.state = rdp->state;
+		stateEvent.active = rdp->state == CONNECTION_STATE_ACTIVE;
+		PubSub_OnConnectionStateChange(context->pubSub, context, &stateEvent);
+	}
+
 	return status;
 }
 
@@ -1589,3 +1606,10 @@ const char* rdp_server_connection_state_string(int state)
 			return "UNKNOWN";
 	}
 }
+
+int rdp_client_get_state(rdpRdp* rdp)
+{
+	if (!rdp)
+		return -1;
+	return rdp->state;
+}
