@@ -383,7 +383,8 @@ static int rpc_client_recv_fragment(rdpRpc* rpc, wStream* fragment)
 				pdu->Type = PTYPE_RESPONSE;
 				pdu->CallId = rpc->StubCallId;
 				Stream_SealLength(pdu->s);
-				rpc_client_recv_pdu(rpc, pdu);
+				if (rpc_client_recv_pdu(rpc, pdu) < 0)
+					return -1;
 				rpc_pdu_reset(pdu);
 				rpc->StubFragCount = 0;
 				rpc->StubCallId = 0;
@@ -983,7 +984,7 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	CopyMemory(&buffer[offset], &request_pdu.auth_verifier.auth_type, 8);
 	offset += 8;
 	Buffers[0].BufferType = SECBUFFER_DATA | SECBUFFER_READONLY; /* auth_data */
-	Buffers[1].BufferType = SECBUFFER_TOKEN; /* signature */
+	Buffers[1].BufferType = SECBUFFER_TOKEN;                     /* signature */
 	Buffers[0].pvBuffer = buffer;
 	Buffers[0].cbBuffer = offset;
 	Buffers[1].cbBuffer = size;
@@ -1022,10 +1023,10 @@ static BOOL rpc_client_resolve_gateway(rdpSettings* settings, char** host, UINT1
 		return FALSE;
 	else
 	{
-		const char* peerHostname = settings->GatewayHostname;
-		const char* proxyUsername = settings->ProxyUsername;
-		const char* proxyPassword = settings->ProxyPassword;
-		*port = settings->GatewayPort;
+		const char* peerHostname = freerdp_settings_get_string(settings, FreeRDP_GatewayHostname);
+		const char* proxyUsername = freerdp_settings_get_string(settings, FreeRDP_GatewayUsername);
+		const char* proxyPassword = freerdp_settings_get_string(settings, FreeRDP_GatewayPassword);
+		*port = freerdp_settings_get_uint32(settings, FreeRDP_GatewayPort);
 		*isProxy = proxy_prepare(settings, &peerHostname, port, &proxyUsername, &proxyPassword);
 		result = freerdp_tcp_resolve_host(peerHostname, *port, 0);
 
@@ -1041,6 +1042,7 @@ static BOOL rpc_client_resolve_gateway(rdpSettings* settings, char** host, UINT1
 
 RpcClient* rpc_client_new(rdpContext* context, UINT32 max_recv_frag)
 {
+	wObject* obj;
 	RpcClient* client = (RpcClient*)calloc(1, sizeof(RpcClient));
 
 	if (!client)
@@ -1081,7 +1083,11 @@ RpcClient* rpc_client_new(rdpContext* context, UINT32 max_recv_frag)
 	if (!client->ClientCallList)
 		goto fail;
 
-	ArrayList_Object(client->ClientCallList)->fnObjectFree = rpc_array_client_call_free;
+	obj = ArrayList_Object(client->ClientCallList);
+	if (!obj)
+		goto fail;
+
+	obj->fnObjectFree = rpc_array_client_call_free;
 	return client;
 fail:
 	rpc_client_free(client);
